#define _POSIX_C_SOURCE 200112L
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

#include "caracteristiques.h"


static char
traiter(mode_t bit, char c1, char c2);

/****************************************************
 * retourne :
 *      0 si OK
 *     -1 si le tampon trop petit
 *     -2 d'autres erreurs
 ****************************************************/
int
caracteristiques(char *nom_fichier, char *pbuffer, size_t taille){
  char *tampon;
  char *p;
  size_t st=256; /*taille tampon*/
  struct stat sstat;
  struct tm *ptm;
  int c=0;/*l'indice dans la tampon*/
  long t;
  char *pc;
  int i;

  if( lstat(nom_fichier, &sstat) == -1 ){
    perror(nom_fichier);
    return -2;
  }

  if( (tampon=malloc(st)) == NULL ){
    return -2;
  }

  /*quel type de fichier*/
  if( S_ISREG(sstat.st_mode) ){
    tampon[c++]='-';
  }
  else if( S_ISFIFO(sstat.st_mode) ){
    tampon[c++]='p';
  }
  else if( S_ISCHR(sstat.st_mode) ){
    tampon[c++]='c';
  }
  else if( S_ISBLK(sstat.st_mode) ){
    tampon[c++]='b';
  }
  else if( S_ISDIR(sstat.st_mode) ){
    tampon[c++]='d';
  }
  else if( S_ISLNK(sstat.st_mode) ){
    tampon[c++]='l';
  }

  tampon[c++]=traiter(sstat.st_mode & S_IRUSR, 'r', '-');
  tampon[c++]=traiter(sstat.st_mode & S_IWUSR, 'w', '-');
  tampon[c++]=traiter(sstat.st_mode & S_IXUSR, 'x', '-');
  tampon[c++]=traiter(sstat.st_mode & S_IRGRP, 'r', '-');
  tampon[c++]=traiter(sstat.st_mode & S_IWGRP, 'w', '-');
  tampon[c++]=traiter(sstat.st_mode & S_IXGRP, 'x', '-');
  tampon[c++]=traiter(sstat.st_mode & S_IROTH, 'r', '-');
  tampon[c++]=traiter(sstat.st_mode & S_IWOTH, 'w', '-');
  tampon[c++]=traiter(sstat.st_mode & S_IXOTH, 'x', '-');


  i=sprintf( &tampon[c], " %8lu %2lu %5lu %5lu %7lu ",
	   (unsigned long) sstat.st_ino, 
	   (unsigned long) sstat.st_nlink, 
	   (unsigned long) sstat.st_uid, 
	   (unsigned long) sstat.st_gid,  
	   (unsigned long) sstat.st_size);
  if(i<0){
    free(tampon);
    return -2;
  }
  c+=i;
  ptm=localtime(&sstat.st_mtime);
  i=sprintf(&tampon[c] , " %04d/%02d/%02d %02d:%02d:%02d ", 
	 ptm->tm_year+1900, ptm->tm_mon, ptm->tm_mday,
	 ptm->tm_hour,ptm->tm_min,ptm->tm_sec);
  if(i<0){
    free(tampon);
    return -2;
  }
  c+=i;

  i=sprintf(&tampon[c],"  ");
  c+=i;


  /*trouver nom de base de fichier (sans chemin 
   *d'acces, sinon l'affichage n'est pas joli)
   */
  pc = nom_fichier + (strlen(nom_fichier)-1);
  while(pc > nom_fichier && *pc != '/'){
    pc--;
  }
  if( *pc == '/' ){
    pc++;
  }
  /* verifier si le tampon est assez grand et
   * agrandir si necessaire                     */
  i=c+strlen(pc)+1;
  if( i>st ){
    /*on ajoute un peu plus pour les liens symboliques*/ 
    st=i+50;    
    p=realloc(tampon,st);
    if(p==NULL){
      free(tampon); 
      return -2;
    }
    tampon=p;
  }

  i=sprintf(&tampon[c],"%s",pc);
  c+=i;

  /* si le fichier est un lien symbolique ajouter ce lien*/
  if( S_ISLNK(sstat.st_mode) ){
    i=sprintf(&tampon[c]," --> ");
    c+=i;

    t = readlink(nom_fichier, &tampon[c], (size_t) st-c-1);
    if( t == -1 ){
      perror(NULL);
      return -2;
    }
    c+=t;
  }
  /*
    if( access(&tampon[c-t],F_OK) == -1 ){
      if( c+11 >= taille ){
	perror("tampon trop petit\n");
	return NULL;
      }

      sprintf(&tampon[c] , " (invalide)");
      c+=11;
      tampon[c]='\0';
    }
  }
  */
  tampon[c++]='\0';
  /* verifier si pbuffer assez grand et 
   * si c'est le cas recopier tampon dedans      */
  if(c>taille){
    free(tampon);
    return -1;
  }
  memcpy(pbuffer,tampon,c);
  free(tampon);
  return 0;
}

static char
traiter(mode_t bit, char c1, char c2){
  if (bit)
    return c1;
  else
    return c2;
}
