#define _POSIX_C_SOURCE 200112L
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "caracteristiques.h"

/****************************************************************
 * parcoursrep -R repertoire
 * permet de faire un parcours recursif du repertoire
 ******************************************************************/

static void parcours(char *nom_rep);

static int recursif=0; /* 0 si listing non recursif *
                        * 1 si recursif             */

int  main(int argc, char *argv[]){

  int i; int k=1;
  
  if(argc == 1){
    parcours(".");
    return EXIT_SUCCESS;
  }
  if( strcmp(argv[1] , "-r")==0 || strcmp(argv[1],"-R")==0 ){
    recursif=1;
    if(argc==2){
      parcours(".");
      return EXIT_SUCCESS;
    }
    k=2;
  }


  for(i=k;i < argc;i++){
    parcours(argv[i]);
  }
  return EXIT_SUCCESS;
}

static void parcours(char *nom_rep){
  /*  struct stat caracteristiques;*/
#define TAILLE_TAMPON 500
  char tampon[TAILLE_TAMPON];
  char chemin[2048];
  DIR *flot;
  int ligne,l;
  struct stat inode;

  if((flot = opendir(nom_rep))==NULL){
    perror(nom_rep);
    return;
  }
  printf("_____________________ %s"
	 " ____________________________________\n",nom_rep);



  /*****************************************************
   *  premiere boucle for pour afficher les caracteristiques des fichiers qui
   *  se trouvent dans le repertoire
   *****************************************************/
  for (;;){
    struct dirent *entree;
    int l;
    errno = 0;
    entree = readdir(flot);/*readdir() retourne NULL si erreur ou
			    *si fin de repertoire, si erreur la variable
			    *errno donne le code d'erreur, si pas d'erreur
			    *errno reste inchange*/
    if (entree == NULL){
      if (errno != 0){
	perror(nom_rep);
	closedir(flot);
	return;
      }
      break;
    }
    if (strcmp(entree->d_name, ".") == 0
	|| strcmp(entree->d_name, "..") == 0)
      continue;

    l=strlen(nom_rep);
    if( nom_rep[l-1]!='/'){
      sprintf(chemin,"%s/%s",nom_rep,entree->d_name);
    }
    else{
      sprintf(chemin,"%s%s",nom_rep,entree->d_name);
    }

    ligne=caracteristiques(chemin, tampon,TAILLE_TAMPON);
    if(ligne == 0){
      printf("%s\n",tampon);
    }
    else if(ligne == -1){
      fprintf(stderr,"%s : tampon trop petit\n","parcours");
    }
  }/*fin for*/
  if( !recursif ){
    closedir(flot);
    return;
  }

  /* revenir au debut de repertoire*/
  rewinddir(flot);
  /*****************************************************
   * duxieme boucle for pour descendre dans les sous repertoires
   *****************************************************/
  for (;;){
    struct dirent *entree;
    errno = 0;
    entree = readdir(flot);
    if (entree == NULL){
      if (errno != 0)
	perror(nom_rep);
      /*      closedir(flot);*/
      break;
    }
    if (strcmp(entree->d_name, ".") == 0
	|| strcmp(entree->d_name, "..") == 0)
      continue;
    l=strlen(nom_rep);

    if( nom_rep[l-1]!='/'){
      sprintf(chemin,"%s/%s",nom_rep,entree->d_name);
    }
    else{
      sprintf(chemin,"%s%s",nom_rep,entree->d_name);
    }

    if (lstat(chemin, &inode) == -1){
      perror(chemin);
      closedir(flot);
      return;
    }
    /* si le fichier est un sous repertoire alors faire un parcours recoursif */
    if (S_ISDIR(inode.st_mode))
      parcours(chemin);
  }/*fin for*/
  closedir(flot);
}


